###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.8 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import Products
import OFS.Folder
import calendar_year
import base_item
import time
import calendar
import todo_category_folder
import utils
import DateTime

calendar_folder_add_redirect = HTMLFile('calendar_folder_add_redirect', globals())
manage_add_calendar_folder_form = HTMLFile('calendar_folder_add', globals())


def manage_add_calendar_folder(self, id, title, REQUEST=None):
    "Add an instance of the calendar_folder class."

    self._setObject(id, calendar_folder(id, title))

    if REQUEST is not None:

        return self.manage_main(self, REQUEST)


class calendar_folder(
	Products.ZCatalog.CatalogAwareness.CatalogAware,
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	base_item.base_item
	):
	"Documentation string"

	meta_type = 'CalendarFolder'
	icon = 'misc_/Calendar/Folder_icon.gif'
	default_catalog = 'Catalog'

	_properties = (
		{'id':'title', 'type':'string'},
		)

	meta_types = (
		{'name':'TodoCategoryFolder', 'action':'manage_add_todo_category_folder_form'},
		{'name':'CalendarYear', 'action':'manage_add_calendar_year_form'},
		)

	manage_add_calendar_year_form = calendar_year.manage_add_calendar_year_form
	manage_add_calendar_year = calendar_year.manage_add_calendar_year
	calendar_year_add_redirect = calendar_year.calendar_year_add_redirect

	manage_add_todo_category_folder_form = todo_category_folder.manage_add_todo_category_folder_form
	manage_add_todo_category_folder = todo_category_folder.manage_add_todo_category_folder

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('Add CalendarYear',
		 ('manage_add_calendar_year', 'manage_add_calendar_year_form')),
		('View CalendarFolders',
		 ('', 'get_years', 'get_ymd', 'get_date', 'display_current_month',
			'get_today', 'display_current_day', 'display_next_day',
			'get_day', 'check_timespan', 'get_unique_id',
			'get_calendar')),
		('Manage Calendar',
			('calendar_folder_add_event', 'manage_add_calendar_folder_event'))
		)

	index_html = HTMLFile('calendar_folder_index', globals())
	calendar_folder_add_event = HTMLFile('calendar_folder_add_event',
						globals())

	product_name = 'CalendarFolder'


	def get_specified_todos(self, REQUEST):
		"""Returns a list of todos selected via REQUEST."""

		# FIXME

		return self.get_todos()


	def get_categories(self):
		"""Returns a list of all the todo categories."""

		categories = []

		for todo_category_folder in self.objectValues('TodoCategoryFolder'):

			categories.extend(todo_category_folder.objectValues('TodoCategory'))

		return categories


	def get_todo_category_folders(self):
		"""Returns a list of all the todo category folders."""

		return self.objectValues('TodoCategoryFolder')


	def get_todos(self):
		"""Returns a list of all the todos."""

		self.ZopeFind(self, obj_metatypes=['Todo'], search_sub=1)


	def render_category_select(self):
		"""Returns a select widget."""

		select_widget = """<select name="category">"""

		for category in self.get_categories():

			select_widget = select_widget + """<option value="%s">%s</option>""" % \
						(category.id, category.title)

		return select_widget + "</select>"


	def render_year_select(self):
		"""Returns a year widget."""

		current_year = self.get_ymd()[0]

		return utils.make_range_select('year', current_year, current_year + 10, pad=1)


	def render_month_select(self):
		"""Returns a month widget."""

		current_month = self.get_ymd()[1]

		return utils.make_range_select('month', 1, 13, selected=current_month, pad=1)


	def render_day_select(self):
		"""Returns a day widget."""

		current_day = self.get_ymd()[2]

		return utils.make_range_select('day', 1, 32, selected=current_day, pad=1)


	def get_years(self):
		"""Returns all year objects contained."""

		calendar_years = self.objectValues('CalendarYear')

		calendar_years.sort(lambda x,y: cmp(int(x.id), int(y.id)))

		return calendar_years


	def get_ymd(self):
		"""Returns a tuple of the current year, month and date."""

		return self.get_date()[:3]


	def get_date(self):
		"""Returns a tuple describing the current time."""

		return time.localtime(time.time())


	def display_current_month(self, details=None):
		"""Renders the current month."""

		year = utils.pad_integer(self.get_date()[0])
		month = utils.pad_integer(self.get_date()[1])

		return self[year][month].display(details=details)


	def get_today(self):
		"""Returns the calendar day representing today."""

		date = self.get_date()

		year = utils.pad_integer(date[0])
		month = utils.pad_integer(date[1])
		day = utils.pad_integer(date[2])

		return self[year][month][day]


	def display_current_day(self):
		"""Renders the current day."""

		return self.get_today().display()


	def display_next_day(self):
		"""Renders the next day."""

		return self.get_day(offset=1).display()


	def get_day(self, offset=0):
		"""Returns a tuple of days which lead the path to the day
		depending of the offset from current day."""

		current_date = DateTime.DateTime()

		new_date = DateTime.DateTime(int(current_date) + (86400 * offset))

		year = utils.pad_integer(new_date.year())
		month = utils.pad_integer(new_date.month())
		day = utils.pad_integer(new_date.day())

		return self[year][month][day]		


	def check_timespan(self, year, month, day,
				start, end, looseness=120):
		"""Checks if given timespan is available.

		year is an integer specifying the year,

		month is an integer specifying the month,

		day is an integer specifying the day,

		start is an integer (in the context of the three variables
		above)	indicating the starting time.

		end is the same as above, only indicating the end
		instead of the start

		loseness is an integer specifying the looseness of
		the possible matching events, in seconds.
		"""

		# This may be faulty, ?FIXME?

		conflicting_events = []

		if len(month) < 2:

			month = '0' + str(month)

		if len(day) < 2:

			day = '0' + day

		if type(year) is type('') and \
		   type(month) is type('') and \
		   type(day) is type(''):

			pass

		else:

			month = str(month)
			day = str(day)
			year = str(year)

		events = self[year][month][day].objectValues('CalendarEvent')

		for event in events:

			print type(start), type(end), type(event.start)

			if event.start + looseness > start and \
			   event.end + looseness > end:

				continue

			if event.start - looseness < start and \
			   event.end - looseness < end:

				continue

			conflicting_events.append(event)

		return conflicting_events


	def manage_add_calendar_folder_event(self, year, month, day,
						REQUEST, RESPONSE):
		"""Adds a calendar event, 'from the root'."""

		day = self[year][month][day]

		day.manage_add_calendar_event(
			REQUEST=REQUEST, RESPONSE=RESPONSE)

		RESPONSE.redirect(day.absolute_url())


	def get_unique_id(self):
		"""Returns a unique identifier."""


		try:

			return self.getParentNode().get_unique_id()

		except AttributeError:

			self.counter = self.counter + 1

			return self.counter


	def __init__(self, id, title=''):
		"Initialize the class instance"

		self.id = id
		self.title = title
		self.counter = 0

		calendar_year.manage_add_calendar_year(self,
			str(self.get_date()[0]), '')


		"""
		OFS.DTMLMethod.addDTMLMethod(self, 'standard_html_header',
			'standard_html_header',
			open('%s/Products/%s/standard_html_header.dtml' % \
				(SOFTWARE_HOME, self.product_name)).read())
										  
		OFS.DTMLMethod.addDTMLMethod(self, 'standard_html_footer',
			'standard_html_footer',
			open('%s/Products/%s/standard_html_footer.dtml' % \
				(SOFTWARE_HOME, self.product_name)).read())

		OFS.DTMLMethod.addDTMLMethod(self, 'standard_css',
			'standard_css',
			open('%s/Products/%s/standard_css.dtml' % \
				(SOFTWARE_HOME, self.product_name)).read())
		"""


	def get_calendar(self):
		"""Returns the calendar folder instance."""

		return self


default__class_init__(calendar_folder)

