###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.8 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import OFS.Folder
import calendar_day
import base_item
import time
import calendar
import utils

calendar_month_add_redirect = HTMLFile('calendar_month_add_redirect',
					globals())
manage_add_calendar_month_form = HTMLFile('calendar_month_add', globals())


def manage_add_calendar_month(self, id, title, REQUEST=None):
    "Add an instance of the calendar_month class"
    self._setObject(id, calendar_month(id, title))

    if REQUEST is not None:

        return self.manage_main(self, REQUEST)


class calendar_month(
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	base_item.base_item
	):
	"Documentation string"

	meta_type = 'CalendarMonth'
	icon = 'misc_/Transform/Folder_icon.gif'

	_properties = (
		{'id':'title', 'type':'string'},
		)

	meta_types = (
		{'name':'CalendarDay', 'action':'manage_add_calendar_day_form'},
		)

	month_names = calendar.month_name
	month_names_abbreviated = calendar.month_abbr

	manage_add_calendar_day_form = calendar_day.manage_add_calendar_day_form
	manage_add_calendar_day = calendar_day.manage_add_calendar_day
	calendar_day_add_redirect = calendar_day.calendar_day_add_redirect

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('Add CalendarDay',
		 ('manage_add_calendar_day', 'manage_add_calendar_day_form')),
		('View CalendarMonths',
		 ('',)),
		)

	index_html = HTMLFile('calendar_month_index', globals())

	def __init__(self, id, title=''):
		"Initialize the class instance"
		self.id = id
		self.title = title


	def render_name(self):
		"""Renders the month name."""

		return self.month_names[int(self.id)]


	def get_context_month(self):

		return self.id


	def get_month_structure(self):
		"""Returns a structure describing the days
		of the current month."""

		return calendar.monthcalendar(int(self.getParentNode().id),
						int(self.id))


	def simple_navigation_html(self):
		"""Returns a simple month navigation list."""

		calendar_month_list = []
		calendar_months = self.get_months()

		for month in calendar_months:

			month_html = """&nbsp;<a href="%s/01/">%s</a>&nbsp;""" % \
					(month.relative_url(),
					month.render_name())

			# Why doesn't "if month is self" work here?

			if month.id == self.id:

				print 'month (%s) is self!' % month.id

				current_month = 1

			else:

				current_month = 0

			calendar_month_list.append({'html':month_html,'is_current_month':current_month})


		return calendar_month_list


	def display(self, details=None, day=None):
		"""Returns an HTML formatted output of the current month."""

		tmp = """<table border="0" cellpadding="%s" cellspacing="%s"
		valign="top" bgcolor="%s" width="100%%">""" % \
			(self.table_cellpadding0,
			self.table_cellspacing0,
			self.color_header1)


		tmp = tmp + """
		<tr>
		<td valign="middle"> Monday </td>
		<td valign="middle"> Tuesday </td>
		<td valign="middle"> Wednesday </td>
		<td valign="middle"> Thursday </td>
		<td valign="middle"> Friday </td>
		<td valign="middle"> Saturday </td>
		<td valign="middle"> Sunday </td>
		</tr>"""


		if day:

			context_year = int(self.getParentNode().id)
			context_month = int(self.id)
			context_day = int(day)

		else:

			context_year, context_month, context_day = None, None, None

		#context_month = int(self.id)
		#context_year = int(self.getParentNode().id)

		#if current_month == context_month:

		#	is_current_month = 1

		#else:

		#	is_current_month = 0

		#if current_year == context_year:

		#	is_current_year = 1

		#else:

		#	is_current_year = 0

		# FIXME

		is_current_year = 1
		is_current_month = 1

		weeks = self.get_month_structure()

		for week_index in range(len(weeks)):

			if week_index % 2:

				bgcolor = self.color_sequence_even0

			else:

				bgcolor = self.color_sequence_odd0

			tmp = tmp + """<tr bgcolor="%s">
			""" % bgcolor

			week = weeks[week_index]
				
			for day in week:

				if day:

					if details:

						event_count = len(self[utils.pad_integer(day)].getChildNodes())

						if event_count:

							formatted_event_count = " (%s)" % event_count

						else:

							formatted_event_count = ""

					else:

						formatted_event_count = ""


					if is_current_month and is_current_year and context_day == day:

						tmp = tmp + """<td align="center" bgcolor="#ffffff"><a href="%s">%s%s</a></td>""" % \
							(self[utils.pad_integer(day)].absolute_url(),
							utils.pad_integer(day), formatted_event_count)

					else:

						tmp = tmp + """<td align="center"><a href="%s">%s%s</a></td>
						""" % (self[utils.pad_integer(day)].absolute_url(),
						utils.pad_integer(day), formatted_event_count)

				else:

					tmp = tmp + """<td>&nbsp;</td>"""

			else:

				tmp = tmp + """</tr>"""

		tmp = tmp + """</table>"""

		return tmp


default__class_init__(calendar_month)

