###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.6 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import Products
import OFS.Folder
from Products.BTreeFolder.BTreeFolder import BTreeFolder
#import message
import base_item
import message_filter


message_container_add_redirect = HTMLFile('message_container_add_redirect', globals())
manage_add_message_container_form = HTMLFile('message_container_add', globals())


def manage_add_message_container(self, title, description, type=[0], REQUEST=None, RESPONSE=None):
	"Add an instance of the message_container class"

	for type in type:

		new_id = str(self.get_unique_id())

		self._setObject(new_id, message_container(new_id, title, description, int(type)))

	if RESPONSE is not None:

		RESPONSE.redirect(self.absolute_url())


class message_container(
	AccessControl.Role.RoleManager,
	Products.ZCatalog.CatalogAwareness.CatalogAware,
	OFS.Folder.Folder,
	#BTreeFolder,
	Persistent,
	Acquisition.Implicit,
	base_item.base_item
	):
	"Documentation string"

	meta_type = 'MessageContainer'
	icon = 'misc_/Transform/Folder_icon.gif'

	_properties = (
		{'id':'title', 'type':'string'},
		{'id':'description', 'type':'text'},
		{'id':'type', 'type':'int', 'select_widget':'types'}
		)
		

	types = {
			0:'Incoming',
			1:'Outgoing',
			2:'Drafts',
			3:'Temporary'
		} 

	_properties = (
		{'id':'title', 'type':'string'},
		)

	meta_types = (
			{'name':'MessageFilter', 'action':'manage_add_message_filter_form'},
			)

	manage_add_message_filter_form = message_filter.manage_add_message_filter_form
	manage_add_message_filter = message_filter.manage_add_message_filter
	message_filter_add_redirect = message_filter.add_message_filter_redirect

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		#('Add Message',
		# ('manage_add_message', 'manage_add_message_orm')),
		('View MessageContainers',
		 ('',)),
		)

	index_html = HTMLFile('message_container_index', globals())
	message_filters = HTMLFile('message_container_message_filters', globals())


	def __init__(self, id, title='', description='', type=0):
		"Initialize the class instance"

		self.id = id
		self.title = title
		self.description = description
		self.type = type

		try:

			self.index_object()

		except AttributeError:

			# No ZCatalog instance available..

			pass


	def get_message_filters(self):
		"""Returns a list of message filters, sorted by priority."""

		message_filters = self.objectValues('MessageFilter')

		message_filters.sort(lambda x,y: cmp(x.priority, y.priority))

		return message_filters


	def get_number_of_messages(self):
		"""Returns the number of messages this object contains."""

		return len(self.objectIds('RFC822Message'))


	def get_number_of_unread_messages(self):
		"""Returns the number of messages unread that this object
		contains."""

		return len(filter(lambda x: not x.read, self.get_messages()))


	def get_messages(self):
		"""Returns a list of messages contained in this message container."""

		return self.objectValues('RFC822Message')


default__class_init__(message_container)

