###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.0 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import Products
import OFS.Folder
import OFS.content_types
import base_item
import string


personal_file_add_redirect = HTMLFile('personal_file_add_redirect', globals())
manage_add_personal_file_form = HTMLFile('personal_file_add', globals())


def manage_add_personal_file(self, file, description, content_type='', REQUEST=None, RESPONSE=None):
	"Add an instance of the personal_file class"

	new_id = str(self.get_unique_id())

	data = file.read()
	filename = file.filename

	backslash_index = string.rfind(filename, '\\')

	# Now, this screws up unix filenames with backslahes
	# in them.. *ponder*

	if backslash_index > -1:

		filename = filename[backslash_index+1:]	

	content_type = content_type or OFS.content_types.guess_content_type(
						name=filename,
						body=data,
						default='application/octet-stream')[0]


	self._setObject(new_id, personal_file(new_id, data, filename, description, content_type))

	if RESPONSE is not None:

		RESPONSE.redirect(self.absolute_url())


class personal_file(
	AccessControl.Role.RoleManager,
	Products.ZCatalog.CatalogAwareness.CatalogAware,
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	base_item.base_item
	):
	"Documentation string"

	meta_type = 'PersonalFile'
	icon = 'misc_/PersonalContainer/Folder_icon.gif'

	_properties = (
		{'id':'name', 'type':'string'},
		{'id':'description', 'type':'text'}
		)

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('View PersonalFile',
		 ('',)),
		)

	index_html = HTMLFile('personal_file_index', globals())


	def __init__(self, id, data, filename, description, content_type):
		"Initialize the class instance"

		self.id = id
		self.data = data
		self.filename = filename
		self.description = description
		self.content_type = content_type

		self.size = len(self.data)

		try:

			self.index_object()

		except AttributeError:

			# No ZCatalog instance available..

			pass


	def download(self, RESPONSE):
		"""Let's the user download the file."""

		RESPONSE.setHeader('Content-Type', '%s; filename=%s' % \
					(self.content_type, self.filename))
		RESPONSE.setHeader('Content-Length', self.size)
		RESPONSE.setHeader('Content-Disposition', 'filename=%s' % self.filename)

		RESPONSE.write(self.data)


default__class_init__(personal_file)

