###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.12 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import Products
import OFS.Folder
import rfc822_attachment
import rfc822_address_container
import rfc822_address
import base_item
import cStringIO
import mimetools
import mimify
import extended_mime_writer
import string
import DateTime
import util

add_rfc822_message_redirect = HTMLFile('rfc822_message_add_redirect',
					globals())
manage_add_rfc822_message_form = HTMLFile('rfc822_message_add', globals())


def manage_add_rfc822_message(self, id, sender='', reply_to='', body='',
				subject='', message_id='', priority=0,
				REQUEST=None):
    "Add an instance of the rfc822_message class"

    self._setObject(id,
			rfc822_message(id, sender, body, subject,
				message_id=message_id, reply_to=reply_to,
				priority=priority))

    if REQUEST is not None:

        return self.manage_main(self, REQUEST)


class dummy_message_container:

	def __init__(self):

		self.id = ''
		self.title = "Don't copy to any folder"


	def absolute_url(self, *args, **kw):

		return ''

	def relative_url(self, *args, **kw):

		return ''


class rfc822_message(
	Products.ZCatalog.CatalogAwareness.CatalogAware,
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	base_item.base_item
	):
	"Documentation string"

	meta_type = 'RFC822Message'
	icon = 'misc_/RFC822Message/Folder_icon.gif'

	priorities = {
			0:'Low',
			1:'Medium',
			2:'High'
		}

	_properties = (
		{'id':'sender', 'type':'string'},
		{'id':'reply_to', 'type':'string'},
		{'id':'body', 'type':'text'},
		{'id':'readonly', 'type':'boolean'},
		{'id':'read', 'type':'boolean'},
		)

	meta_types = (
		{'name':'RFC822Attachment',
		 'action':'manage_add_rfc822_attachment_form'},
		{'name':'RFC822Address',
		 'action':'manage_add_rfc822_address_form'},
		)

	manage_add_rfc822_attachment_form = rfc822_attachment.manage_add_rfc822_attachment_form
	manage_add_rfc822_attachment = rfc822_attachment.manage_add_rfc822_attachment
	rfc822_attachment_add_redirect = rfc822_attachment.rfc822_attachment_add_redirect

	manage_add_rfc822_address_container_form = rfc822_address_container.manage_add_rfc822_address_container_form
	manage_add_rfc822_address_container = rfc822_address_container.manage_add_rfc822_address_container
	rfc822_address_container_add_redirect = rfc822_address_container.rfc822_address_container_add_redirect


	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('Add RFC822Attachment',
		 ('manage_add_rfc822_attachment', 'manage_add_rfc822_attachment_form')),
		('Add RFC822Address',
		 ('manage_add_rfc822_address', 'manage_add_rfc822_address_form')),
		('View RFC822Messages',
		 ('',)),
		)

	index_html = HTMLFile('rfc822_message_index', globals())

	view_index_html = HTMLFile('rfc822_message_view_index', globals())
	edit_index_html = HTMLFile('rfc822_message_edit_index', globals())
	edit_attachments_html = HTMLFile('rfc822_message_edit_attachments', globals())

	def mark_as_read(self):
		"""Marks the message as read."""

		self.read = 1


	def copy(self, folder=None):
		"""Copies message to temporary folder."""

		if folder is not None:

			folder = folder

		#else:

		#	temporary = self.get_temporary_folder()

		message_copy = self.getParentNode().manage_copyObjects(self.id)
		folder.manage_pasteObjects(cb_copy_data=message_copy)

		# We do this so we never get id conflicts.
		# We may get it if it's copied to the same container; FIXME

		new_id = str(self.get_unique_id())

		folder.manage_renameObject(self.id, new_id)

		message_new = folder[new_id]

		message_new._raw_body = 'No raw body available..'

		return message_new


	def delete_address_fields(self):
		"""Removes all recipients."""

		for address_field in ['to', 'cc', 'bcc']:

			for address in self[address_field].objectValues():

				self[address_field].manage_delObjects(address.id)


	def forward(self, REQUEST=None, RESPONSE=None):
		"""Method that prepares a message for forwarding."""

		if REQUEST.has_key('temporary_folder'):

			temporary_folder = REQUEST.resolve_url(REQUEST['temporary_folder'])

		else:

			temporary_folder = self.get_temporary_folder()

		new_id = str(self.get_unique_id())

		temporary_folder.manage_add_rfc822_message(new_id, sender=self.email_address,
							body=self.render_forward_body(),
							subject=self.render_forward_subject())

		new_message = temporary_folder[new_id]

		if self.get_attachments():

			for attachment in self.get_attachments():

				new_id = str(self.get_unique_id())

				new_message.manage_add_rfc822_attachment(new_id,
									data=attachment.data,
									filename=attachment.filename,
									content_type=attachment.content_type)
								

		RESPONSE.redirect(new_message.absolute_url())



	def reply(self, REQUEST=None, RESPONSE=None, all=None):
		"""Method that prepares a message for replying."""

		new_id = str(self.get_unique_id())

		new_body = self.render_reply_body()

		if self.headers.has_key('references'):

			references = string.split(self.headers['references'])

		else:

			references = []

		headers = {}

		try:

			references.append(self.headers['message-id'])

		except KeyError:

			# There was no message id..

			pass

		if references:

			headers['references'] = string.join(references, ' ')

		if REQUEST is not None and REQUEST.has_key('temporary_folder'):

			# FIXME

			temporary_folder = REQUEST.resolve_url(
						REQUEST['temporary_folder'])

		else:

			temporary_folder = self.get_temporary_folder()

		manage_add_rfc822_message(temporary_folder, new_id,
					  sender=temporary_folder.email_address,
					  body=new_body,
					  subject=self.render_reply_subject())

		temporary_folder[new_id]['to'].manage_add_rfc822_address(
			str(self.get_unique_id()), self.sender, comment='', realname='')

		temporary_folder[new_id].references = references[:]

		if all is not None:

			for address_field in self.get_filled_address_fields():
				
				for rfc822_address in self[address_field].objectValues(
					'RFC822Address'):

					temporary_folder[new_id]['cc'].manage_add_rfc822_address(
						str(self.get_unique_id()), rfc822_address.address,
						comment=rfc822_address.comment,
						realname=rfc822_address.realname)

			temporary_folder[new_id].newsgroups = self.newsgroups[:]

		RESPONSE.redirect(temporary_folder[new_id].absolute_url())


	def reply_all(self, REQUEST=None, RESPONSE=None):
		"""Method that prepares a message for replying."""

		self.reply(REQUEST=REQUEST, RESPONSE=RESPONSE, all=1)


	def render_reply_subject(self):
		"""Method for rendering an appropriate subject upon replies."""

		subject = self.headers['subject'] or self.subject

		if string.lower(subject)[:3] == 're:':

			return subject

		else:

			return 'Re: ' + subject


	def render_forward_subject(self):
		"""Method for rendering an appropriate subject upon forwards."""

		subject =self.headers['subject'] or self.subject

		if string.lower(subject)[:4] == 'fwd:':

			return subject

		else:

			return 'Fwd: ' + subject


	def render_html_subject(self):

		subject = self.render_subject()

		if self.priority > 0:

			if self.priority == 1:

				subject = '<span style="color: orange;">%s</span>' % subject

			if self.priority == 2:

				subject = '<span style="color: red;">%s</span>' % subject


		return subject


	def render_subject(self):
		"""Returns the subject."""

		subject = ''

		if self.headers.has_key('subject'):

			subject = string.strip(self.headers['subject'])

		if not subject:

			try:

				subject = string.strip(self.subject)

			except AttributeError:

				pass

		if not subject:

			subject = '[No subject]'

		return subject


	def render_reply_body(self):
		"""Method for rendering an appropriate subject upon replies."""

		return '> ' + string.join(string.split(self.body,'\n'), '\n> ')


	def render_forward_body(self):
		"""Quotes the body of the forwarded message."""

		new_body = string.split(self.body, '\n')

		new_body = '>> ' + string.join(new_body, '\n>> ')

		return new_body
									   

	def render_prioritize_widget(self):
		"""Returns an HTML widget for prioritizing."""

		select_widget = """<select name="priority:int">"""

		priority_keys = self.priorities.keys()
		priority_keys.sort()

		for priority in priority_keys:

			if priority == self.priority:

				selected = " selected"

			else:

				selected = ""

			select_widget = select_widget + """<option value="%s"%s>%s</option>""" % \
				(priority, selected, self.priorities[priority])

		return select_widget + """</select>"""


	def render_move_to_widget(self):
		"""Returns an HTML widget for move-to folder selection."""

		select_widget = """<select name="message_container:string">"""

		for message_container in self.get_message_containers(type=0):

			select_widget = select_widget + """<option value="%s">%s</option>""" % \
				(message_container.absolute_url(), message_container.title)

		return select_widget + "</select>"


	def render_copy_to_header_selection(self):
		"""Returns an HTML widget for copy-to-folder selection."""

		select_widget = """<select name="copy-to-folder:string">"""

		for message_container in [dummy_message_container()] + self.get_message_containers(type=1):

			if message_container.relative_url() == self.headers['copy-to-folder']:

				selected = "selected"

			else:

				selected = ""

			select_widget = select_widget + """<option value="%s" %s>%s</option>""" % \
							(message_container.relative_url(),
								selected,
								message_container.title or \
									message_container.id)


		return select_widget + "</select>"


	def render_attachment_count(self):
		"""Returns a string indicating the number of attachments."""

		number_of_attachments = len(self.get_attachments())

		if number_of_attachments:

			return '(%s)' % number_of_attachments

		else:

			return ''


	def render_date(self):
		"""Returns a formatted view of the message date."""

		try:

			return DateTime.DateTime(self.headers['date']).Date()

		except KeyError:

			return 'No date specified'

		except DateTime.DateTime.DateTimeError, value:

			return 'Internal error (%s)' % (DateTime.DateTimeError, value)

		except DateTime.DateTime.SyntaxError, value:

			return 'Message date was malformed'

		except Exception, value:

			return 'Unknown exception (%s)' % value


	def get_filled_address_fields(self):
		"""Returns address fields that contain one or more addresses."""

		address_fields = []

		for address_field in ['to', 'cc', 'bcc']:

			if len(self[address_field].objectIds('RFC822Address')):

				address_fields.append(address_field)

		return address_fields


	def render_sender(self):
		"""Renders the 'from' address field."""

		return self.sender


	def render_newsgroups(self):
		"""Renders the newsgroups header field."""

		return string.join(self.newsgroups, ', ')


	def render_address_field(self, field='to'):
		"""Renders the addresses contained in the given field."""

		#return_string = ""
		
		addresses = map(lambda x: x.address, self[field].objectValues())
		addresses = filter(None, addresses)


		#for object in self[field].objectValues():

		#	return_string = return_string + '%s, ' % object.address

		#else:

		#	return_string = return_string[:-2]

		return string.join(addresses, ', ')


	def render_html_priority(self):
		"""Returns an HTML formatted presentation of the priority."""

		priority = self.render_priority()

		if self.priority > 0:

			if self.priority == 1:

				'<span style="color: orange;">%s</span>' % priority

			if self.priority == 2:

				'<span style="color: red;">%s</span>' % priority

		return priority
		

	def render_priority(self):
		"""Returns a text presentation of the priority."""

		return self.priorities[self.priority]


	def move_message(self, REQUEST, RESPONSE):
		"""Moves self to another container."""

		current_message_container_url = self.getParentNode().absolute_url()

		cut_data = self.getParentNode().manage_cutObjects(self.id)

		message_container = REQUEST.resolve_url(REQUEST['message_container'])
		message_container.manage_pasteObjects(cb_copy_data=cut_data)

		RESPONSE.redirect(current_message_container_url)


	def prioritize(self, REQUEST, RESPONSE):
		"""Prioritizes the message."""

		self.priority = REQUEST['priority']

		RESPONSE.redirect(self.absolute_url() + '/')


	def update(self, REQUEST=None, RESPONSE=None):
		"""Updates message data."""

		self.body = REQUEST['body']
		self.headers['subject'] = REQUEST['subject']
		self.headers['copy-to-folder'] = REQUEST['copy-to-folder']
		newsgroups = map(string.strip, string.split(REQUEST['newsgroups'], ','))
		self.newsgroups = filter(None, newsgroups)
		self.priority = REQUEST['priority']

		to = map(string.strip, string.split(REQUEST['to'], ','))
		cc = map(string.strip, string.split(REQUEST['cc'], ','))
		bcc = map(string.strip, string.split(REQUEST['bcc'], ','))

		for address_field in ((to,'to'), (cc,'cc'), (bcc,'bcc')):

			existing_addresses = self[address_field[-1]].objectValues()
			request_addresses = address_field[0]

			for rfc822_address in existing_addresses:

				if not rfc822_address.address in address_field[0]:

					self[address_field[-1]].manage_delObjects(rfc822_address.id)

			"MARK"

			for address in request_addresses:

				for existing_address in existing_addresses:

					if address == existing_address.address:

						break

				else:

					self[address_field[-1]].manage_add_rfc822_address(
						str(self.get_unique_id()), address)


		if RESPONSE is not None:

			#oops_it_happened_again = "http://10.1.1.233:8080/Transform/Consultants/morten/Mail/Temporary/233/"

			RESPONSE.redirect(self.absolute_url() + '/')			


	def save_draft(self, REQUEST=None, RESPONSE=None):
		"""Saves a draft of this message."""

		self.update(REQUEST=REQUEST)

		draft_folder = self.get_draft_folder()

		copy_data = self.getParentNode().manage_copyObjects(self.id)
		draft_folder.manage_pasteObjects(copy_data)

		return 'draft saved..'


	def raw_body(self, RESPONSE):
		"""Returns the original message body."""

		RESPONSE.setHeader('content-type', 'text/plain')

		return self._raw_body


	def send(self, REQUEST, RESPONSE=None):
		"""Sends the message."""

		if REQUEST is not None:

			self.update(REQUEST=REQUEST)

		# Do message format magic and such calls here

		if not self.get_filled_address_fields() and not self.newsgroups:

			raise ValueError, 'No recipients or newsgroups specified'

		self.headers['date'] =  self.ZopeTime().rfc822()

		self.headers['message-id'] = '<' + str(self.get_unique_id()) + '.' + \
					mimetools.choose_boundary() + '@' + \
					string.split(self.email_address, '@')[-1] + '>'
									   

		if self.headers['copy-to-folder']:

			copy_to_folder = self.resolve_relative_url(self.headers['copy-to-folder'],
									REQUEST=REQUEST)

			reference = self.copy(folder=copy_to_folder)

			reference.readonly = 1

		# We don't want to render the same message twice..
		# Could be very expensive.

		raw_message = self.raw_message()

		if self.get_filled_address_fields():

			res = self.post(self, raw_message=raw_message)

		if filter(None, self.newsgroups):

			news_host = self.get_news_host()
			
			news_host.post(self, raw_message=raw_message)


		if RESPONSE:

			# What screen do we return to here?

			RESPONSE.redirect(self.get_gum().absolute_url())

			#RESPONSE.setHeader('content-type', 'text/plain')
			#return self.raw_message()
				   


	def add_recipient(address_field='to', address='', realname='', comment=''):
		"""Adds a recipient to the given address field."""

		new_id = str(self.get_unique_id())

		self[address_field].manage_add_rfc822_address(new_id, address,
							comment=comment,
							realname=realname)


	def set_header(self, header_name, header_value):
		"""Sets a mail message header."""

		self.headers[header_name] = header_value


	def add_newsgroup(self, newsgroup):
		"""Adds a newsgroup to the message."""

		try:

			self.newsgroups.index(newsgroup)
			return

		except ValueError:

			self.newsgroups.append(newsgroup)


	def get_recipients(self):
		"""Returns a list of addresses."""

		recipient_list = []

		for address_field in self.get_filled_address_fields():

			for rfc822_address in self[address_field].objectValues(
				'RFC822Address'):

				recipient_list.append(rfc822_address.address)

		return recipient_list


	def get_attachments(self):
		"""Returns a list of RFC822Attachment objects."""

		return self.objectValues('RFC822Attachment')


	def raw_message(self):
		"""Renders an RFC822 compliant message."""

		rendered_message = cStringIO.StringIO()

		writer = extended_mime_writer.extended_mime_writer(rendered_message)

		writer.addheader('From', self.sender)

		for header_key in self.headers.keys():

			if header_key == 'copy-to-folder':

				continue

			writer.addheader(header_key, self.headers[header_key])

		writer.addheader('X-Priority', self.render_priority())

		for address_field in self.get_filled_address_fields():

			if address_field != 'bcc':
			
				writer.addheader(
					string.capitalize(address_field),
					mimify.mime_encode_header(
						self.render_address_field(
							field=address_field)))

		if self.newsgroups:

			writer.addheader('Newsgroups', self.render_newsgroups())

		

		#else:

		#	raise 'MessageIDException', 'No message-id specified'

		if self.references:

			writer.addheader('References', string.join(self.references, ' '))
		
		if self.get_attachments():

			writer.addheader('Mime-Version', '1.0')

		writer.addheader('X-Mailer', '%s (%s), %s' % \
			('Zope GUM', self.get_gum_version(), self.get_gum_website()))

		writer.flushheaders()

		if self.get_attachments():

			writer.startmultipartbody('mixed')

			mime_part = writer.nextpart()

			mime_part_writer = mime_part.startbody('text/plain')

			mimetools.encode(cStringIO.StringIO(self.body),
						 mime_part_writer, 'quoted-printable')

			for attachment in self.get_attachments():

				mime_part = writer.nextpart()

				mime_part.addheader('Content-Transfer-Encoding',
							'base64')

				# ?FIXME?
				# Let the attachment encoding decide?

				tmp_headers = {'content-transfer-encoding':
						'base64'}
				tmp_headers.update(attachment.headers)

				mime_part_writer = mime_part.startbody(
						attachment.content_type,
						[('filename', attachment.filename)],
						tmp_headers)

				mime_part_writer.write(attachment.encode())

			else:

				writer.lastpart()

		else:

			# We have to do this to add headers..

			#for header_name in self.headers.keys():

			#	writer._fp.write('%s: %s\n' % (header_name, self.headers[header_name]))

			writer._fp.write('Content-Transfer-Encoding: quoted-printable\n')
			writer._fp.write('Subject: %s\n' % self.headers['subject'])

			body_writer = writer.startbody('text/plain', [], # FIXME
						{'Content-Transfer-Encoding':
						'quoted-printable'})

			body = cStringIO.StringIO(self.body)
			body.seek(0)

			mimetools.encode(body, body_writer, 'quoted-printable')

		return rendered_message.getvalue()


	def get_newsgroups(self):
		"""Returns a list of newsgroups."""

		return self.newsgroups


	def get_size(self):
		"""Returns the total size.  Isn't entirely accurate as we don't include headers."""

		size = len(self.body)

		for attachment in self.get_attachments():

			size = size + attachment.size

		return size


	def render_size(self, details=None):
		"""Returns the size in a human-readable format."""

		return util.format_size(self.get_size(), details=details)


	def get_navigation_title(self):
		"""See below."""

		return self.render_subject()


	def name_title_or_id(self):
		"""Returns an appropriate subject, for use in the navigation."""

		return self.get_navigation_title()


	def __init__(self, id, sender, body, subject, message_id='', reply_to='', priority=0):
		"Initialize the class instance"

		self.id = id

		self.sender = sender
		self.reply_to = reply_to
		self.body = body
		#self.subject = subject
		self.message_id = message_id

		self.read = 0
		self.priority = priority

		self.newsgroups = []
		self.headers = {'subject':subject, 'copy-to-folder':''}

		# FIXME?
		self.references = []

		self.manage_add_rfc822_address_container('to', title='to')
		self.manage_add_rfc822_address_container('cc', title='cc')
		self.manage_add_rfc822_address_container('bcc', title='bcc')

		try:

			self.index_object()

		except AttributeError:

			# No ZCatalog instance available..

			pass


default__class_init__(rfc822_message)


